/*
 *= Rohde&Schwarz Digital Radiocommunication Tester Program Example ===========
 *
 * C Example Program 3: RF Non Signalling Measurements
 *
 * The following example explains how to handle primary and secondary
 * addressing and how to perform a simple measurement with the CMU. In the
 * example, remote control via GPIB bus and the programming language C is used.
 * 
 * 
 * GSM uses the GMSK modulation scheme with a symbol rate of 270.833 kBit/s and
 * four different phases that can be occupied during one symbol interval. If a
 * constant series of zeros or ones is transmitted, the rotation of the I/Q
 * vector causes a side band at 270.833/4 Hz = 667.70825 Hz from the carrier.
 * Compared to the side band, the original carrier frequency is suppressed by
 * -30 dB to -40 dB. The following example program measures the carrier signal,
 * the upper and the lower side band using the RF Spectrum and the RF Power vs
 * Time measurement groups. No additional options are needed.
 *
 * Compiler name and release(s) : LabWindows/CVI 5.0
 * Original Release : January, 2000
 *
 * By ROHDE & SCHWARZ FRANCE:
 *      CONTACTS :
 *              Support & Application
 *              Rohde & Schwarz France
 *              Parc Tertiaire de Meudon
 *              25-27, rue Jeanne Braconnier
 *              92366 Meudon-la-Foret
 *              Telephone : 00 33 1 41 36 10 29
 *              Fax       : 00 33 1 41 36 11 10
 *              e-Mail    : GSA@rsf.rohde-schwarz.com
 *
 * Version : 1.03
 *  for CVI and c++
 *
 * Copyright 2000 ROHDE & SCHWARZ FRANCE. All Rights Reserved.
 *
 * ----------------------------------------------------------------------------
 *                         MODIFICATON HISTORY
 * ----------------------------------------------------------------------------
 *  Date      | Author      | Description
 *    Release |     Company |            
 * ----------------------------------------------------------------------------
 *  20.Jul.00 | S.Blanchon  | Creation (accordingly to the Operating Manual
 *       V1.0 |         RSF | Edition 04).
 *            |             | 
 *            |             | Delivery version 1.0
 * ----------------------------------------------------------------------------
 *  10.Aug.00 | J.Sima      | Added a call to RSCMU_RF_NSig_SourRfgLev function
 *       V1.01|             | in section RF GENERATOR SETTINGS in order to
 *            |             | provide correct RF Level of the outpur signal.
 *            |             | 
 *            |             | Delivery version 1.01
 * ----------------------------------------------------------------------------
 *  19.Mar.01 | J.Sima      | Modifications:
 *       V1.02|             | 	 Changed the data type of local variable l_vStartTime
 *            |             |    to ViReal64, so that it can be passed by reference to
 *            |             |    RSCMU_RF_NSig_ReadArrPow as a second parameter.
 *            |             |
 *            |             | Delivery version 1.02
 * ----------------------------------------------------------------------------
 *  19.Juni.01| W.Klotzsche | Modifications:
 *       V1.03|             | 	 Changed for useing in CVI and VC
 *            |             |
 *            |             | Delivery version 1.03
 *=============================================================================

 */
	#include <stdio.h>

#ifdef _CVI_
	//for CVI
	#include <utility.h>
	#include <ansi_c.h>
	#include <string.h>
	#include <formatio.h>
	#include <visa.h>
	//#include <userint.h>
	#include <cvirte.h> /* Needed if linking in external compiler; harmless otherwise */
#endif

// Include R&S CMU driver header file
#include "RSCMU200.h" 

// Variable declaration for secondary address for Base definition and for
// Function Groups definition
static ViSession    g_hRsCmuBase  = VI_NULL;        // Basic Unit
static ViSession    g_hRsCmuRf_NSig = VI_NULL;      // RF Non-Signalling Unit

// Macro "CheckCmuError" definition
#ifndef CheckCmuError
#define CheckCmuError(Code) if (_error = (DoCheckCmuError(__FILE__,__LINE__,Code)), (_error = (_error < 0) ? _error : 0)) \
  {goto Error;} else
#endif

// Macro "WriteCmuViInt32Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViInt32Value
#define WriteCmuViInt32Value(String,ViInt32Value) \
(ViInt32Value != RSCMU_NAN_VI_INT32) ? printf ("%s%d\n", String, ViInt32Value) : printf ("%sNot a number\n", String);
#endif

// Macro "WriteCmuViReal64Value" definition (Check if no result could be
// obtained (yet), or if the result obtained is invalid).
#ifndef WriteCmuViReal64Value
#define WriteCmuViReal64Value(String,ViReal64Value) \
(ViReal64Value != RSCMU_NAN) ? printf ("%s%lf\n", String, ViReal64Value) : printf ("%sNot a number\n", String);
#endif


// ... function definition for the previous macro
int DoCheckCmuError (char *File, int Line, ViStatus CmuStatus)
{
  char l_vCmuErrMsg[256];
  char l_vErrBuf[512];
//   ViInt32 _error;
    
    if (CmuStatus != VI_SUCCESS)
    {
        RSCMU_Base_errorMessage (VI_NULL, CmuStatus, l_vCmuErrMsg);
        sprintf (l_vErrBuf, "Error in file %s, Line %d: %s", File, Line, l_vCmuErrMsg);

#ifdef _CVI_
		MessagePopup ("ERROR MESSAGE (CMU)", l_vErrBuf);
#else
		printf ("\nERROR MESSAGE (CMU) %s\n\n", l_vErrBuf);

#endif
   
    return -1;
  }
  return 0;
}


/*****************************************************************************/
/*======================== Main =============================================*/
/*****************************************************************************/
int main (int argc, char *argv[])
{                                  
  ViInt32    _error;
  
  ViChar     l_vInstrDriverRev[256];
  ViChar     l_vInstrFirmwareRev[256];
  
  ViChar     errorMessage[256];
  
  ViReal64   l_vStartFrq;
  ViInt32    l_vSamp;
  ViReal64   l_vArrSpec[560];
  
  ViReal64   l_vStartTime;
  ViReal64   l_vArrPowerVsTime[500];
   
 // only for CVI 
#ifdef _CVI_
  if (InitCVIRTE (0, argv, 0) == 0) /* Needed if linking in external compiler; harmless otherwise */
 //   return -1; /* out of memory */ 
 Cls();
#endif
  
  // ==========================================================================
  // CMU BASIC UNIT
  // --------------------------------------------------------------------------
  // BASIC UNIT INITIALISATION
  //  -> GPIB board index is assigned to 0
  //  -> Primary address to the GPIB bus is assigned to the default (20)
  //  -> Ask for the indentifier of the CMU, reset the instrument
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_Base (&g_hRsCmuBase, RSCMU_GPIB, 0, 20, RSCMU_DO_QUERY, RSCMU_RESET_ON, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS CONFIGURATION
  //   Subsystem Remote
  //     -> Secondary addresses of the GPIB bus are assigned for RF function
  //        group
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Base_SystRemAddrSec (g_hRsCmuBase, 1, RSCMU_RF_NSIG, errorMessage));
  
  // --------------------------------------------------------------------------
  // FUNCTION GROUPS INITIALISATION
  // --------------------------------------------------------------------------
  CheckCmuError (RSCMU_Init_RF_NSig (g_hRsCmuBase, &g_hRsCmuRf_NSig, errorMessage));
  
  
  CheckCmuError (RSCMU_Base_RevisionQuery (g_hRsCmuBase, l_vInstrDriverRev, l_vInstrFirmwareRev, errorMessage));
  
  // --------------------------------------------------------------------------
  printf ("Instrument Driver Revision  (Base): %s\n", l_vInstrDriverRev);
  printf ("Instrument Firmware Revision      : %s\n", l_vInstrFirmwareRev);
  printf ("\n");
  
  // Opens the file "RsCmu.iee" for output IEEE bus commands.
  CheckCmuError (RSCMU_Trace_ON (g_hRsCmuBase, "RsCmu.iee", errorMessage));
  
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // CONNECTORS + ATTENUATION
  //   Define input connector RF2 and output connector RF2.
  //   Make sure that external attenuations are set to zero.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_Inp (g_hRsCmuRf_NSig, RSCMU_INPUT_RF2, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_Outp (g_hRsCmuRf_NSig, RSCMU_OUTPUT_RF2, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SensCorrLossInp (g_hRsCmuRf_NSig, RSCMU_INPUT_RF2, 0.0, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SourCorrLossOutp (g_hRsCmuRf_NSig, RSCMU_OUTPUT_RF2, 0.0, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // RF GENERATOR SETTINGS
  //   Set RF carrier frequency to 900.0 MHz
  //   Switch on frequency offset (single side band modulation)
  //   Set offset frequency to 67.7 kHz.
  //   And switch on RF generator.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_SourRfgFreq (g_hRsCmuRf_NSig, 900.0, RSCMU_MHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SourRfgMod (g_hRsCmuRf_NSig, RSCMU_SSB, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SourRfgModSsbFreq (g_hRsCmuRf_NSig, 67700, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SourRfgLev (g_hRsCmuRf_NSig, -20.0, errorMessage));
  
  CheckCmuError (RSCMU_RF_NSig_InitRfg (g_hRsCmuRf_NSig, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  //    Set the input level to manual setting
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_SensLevMode (g_hRsCmuRf_NSig, RSCMU_MANUAL, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SPECTRUM ANALYZER SETTINGS
  //   Set center frequency to generator frequency (900.0 MHz) for spectrum measurement
  //   Set frequency span to 500 kHz
  //   Set resolution bandwidth of the spectrum analyzer
  //   Single shot measurement, no stop on error.
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_SensSpecFreqCentSpan (g_hRsCmuRf_NSig, 900.0, RSCMU_MHZ, 0.5, RSCMU_MHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SensSpecFreqBand (g_hRsCmuRf_NSig, RSCMU_20KHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ConfSpecContRep (g_hRsCmuRf_NSig, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  
  printf ("\tSpectrum Measurement\n");
  printf ("\t--------------------\n");
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SPECTRUM MEASUREMENT at 900 MHz + Offset 67.7 kHz, 1 point
  //   Select single measurement point at 900.0677 MHz
  //   Start single shot spectrum measurement, wait until it is terminated, and
  //   return result (1 value)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_ConfArrSpecRang (g_hRsCmuRf_NSig, 900.0677, RSCMU_MHZ, 1, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ReadArrSpec (g_hRsCmuRf_NSig, &l_vStartFrq, &l_vSamp, l_vArrSpec, errorMessage));
  WriteCmuViReal64Value ("\tPower at 900.0677 MHz  [dBm] = ", l_vArrSpec[0]);
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SPECTRUM MEASUREMENT at 900 MHz
  //   Select single measurement point at 900.0 MHz
  //   Start single shot spectrum measurement, wait until it is terminated, and
  //   return result (1 value)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_ConfArrSpecRang (g_hRsCmuRf_NSig, 900.0, RSCMU_MHZ, 1, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ReadArrSpec (g_hRsCmuRf_NSig, &l_vStartFrq, &l_vSamp, l_vArrSpec, errorMessage));
  WriteCmuViReal64Value ("\tPower at 900.0000 MHz  [dBm] = ", l_vArrSpec[0]);
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // SPECTRUM MEASUREMENT at 900 MHz - Offset 67.7 kHz, 1 point
  //   Select single measurement point at 899.9323 MHz
  //   Start single shot spectrum measurement, wait until it is terminated, and
  //   return result (1 value)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_ConfArrSpecRang (g_hRsCmuRf_NSig, 899.9323, RSCMU_MHZ, 1, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ReadArrSpec (g_hRsCmuRf_NSig, &l_vStartFrq, &l_vSamp, l_vArrSpec, errorMessage));
  WriteCmuViReal64Value ("\tPower at 899.9323 MHz  [dBm] = ", l_vArrSpec[0]);
  
  CheckCmuError (RSCMU_RF_NSig_AborSpec (g_hRsCmuRf_NSig, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // PRODUCE A RAMPED SIGNAL
  //   Select pulsed signal (as opposed to CW)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_SourRfgPulsStat (g_hRsCmuRf_NSig, RSCMU_ON, errorMessage));
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // CONFIGURE THE POWER/T MEASUREMENT
  //   Set center frequency for power vs. time measurement
  //   Set resolution bandwidth of the RF analyzer
  //   Set evaluation time of power measurement
  //   Trigger measurement by IF signal level (recommended)
  //   Set expected maximum input level
  //   Select single measurement point at 0.3 ms
  //   Single shot measurement, no stop on error
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_SensPowFreqCent (g_hRsCmuRf_NSig, 900.0677, RSCMU_MHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SensPowFreqBand (g_hRsCmuRf_NSig, RSCMU_20KHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SensPowTimeSpan (g_hRsCmuRf_NSig, 1, RSCMU_MS, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_TrigSour (g_hRsCmuRf_NSig, RSCMU_IFP, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_SensLevMax (g_hRsCmuRf_NSig, -10.0, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ConfArrPowRang (g_hRsCmuRf_NSig, 0.3, RSCMU_MS, 1, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ConfPowContRep (g_hRsCmuRf_NSig, RSCMU_SINGLE_SHOT, RSCMU_NONE, RSCMU_NONE, errorMessage));
  
  printf ("\tPower vs. Timme measurement\n");
  printf ("\t---------------------------\n");
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // POWER VS. TIME MEASUREMENT at 900.0677 MHz, 1 point
  //   Start single shot measurement, wait until it is terminated, and return
  //   return result (1 value)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  CheckCmuError (RSCMU_RF_NSig_ReadArrPow (g_hRsCmuRf_NSig, &l_vStartTime, &l_vSamp, l_vArrPowerVsTime, errorMessage));
  WriteCmuViReal64Value ("\tPower at 900.0677 MHz  [dBm] = ", l_vArrPowerVsTime[0]);
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // POWER VS. TIME MEASUREMENT at 900.0 MHz, 1 point
  //   Select single measurement point at 900.0 MHz 
  //   Start single shot measurement, wait until it is terminated, and return
  //   return result (1 value)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
  CheckCmuError (RSCMU_RF_NSig_SensPowFreqCent (g_hRsCmuRf_NSig, 900.0, RSCMU_MHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ReadArrPow (g_hRsCmuRf_NSig, &l_vStartTime, &l_vSamp, l_vArrPowerVsTime, errorMessage));
  WriteCmuViReal64Value ("\tPower at 900.0000 MHz  [dBm] = ", l_vArrPowerVsTime[0]);
  
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
  // POWER VS. TIME MEASUREMENT at 900 MHz - Offset 67.7 kHz, 1 point 
  //   Select single measurement point at 899.9323 MHz 
  //   Start single shot measurement, wait until it is terminated, and return
  //   return result (1 value)
  // ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
  CheckCmuError (RSCMU_RF_NSig_SensPowFreqCent (g_hRsCmuRf_NSig, 899.9323, RSCMU_MHZ, errorMessage));
  CheckCmuError (RSCMU_RF_NSig_ReadArrPow (g_hRsCmuRf_NSig, &l_vStartTime, &l_vSamp, l_vArrPowerVsTime, errorMessage));
  WriteCmuViReal64Value ("\tPower at 899.9323 MHz  [dBm] = ", l_vArrPowerVsTime[0]);
  
  // Close the file "RsCmu.iee".
  // CheckCmuError (RSCMU_Trace_OFF (g_hRsCmuBase, errorMessage));
  
  CheckCmuError (RSCMU_Close_RF_NSig (g_hRsCmuRf_NSig, errorMessage));
  CheckCmuError (RSCMU_Close_Base (g_hRsCmuBase, errorMessage));
 
  return 0;

Error:
  return -1;
}

